package org.richfaces;

import org.ajax4jsf.test.base.SeleniumTestBase;
import org.ajax4jsf.test.base.Templates;
import org.testng.annotations.AfterTest;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.Parameters;
import org.testng.annotations.Test;

public class AjaxRegionTest extends SeleniumTestBase {

    public AjaxRegionTest() {
        super("http", "localhost", "8080");
    }

    /**
     * This method are invoking before selenium tests started
     */
    @BeforeTest
    @Parameters( { "browser" })
    public void startSelenium(String browser) {
        super.startSelenium(browser);
    }

    /**
     * This method are invoking after selenium tests completed
     */
    @AfterTest
    public void stopSelenium() {
        super.stopSelenium();
    }

    @Test
    public void testAjaxRegionComponent() throws Exception {
    	_testAjaxRegionComponent(Templates.SIMPLE);
        _testAjaxRegionComponent(Templates.DATATABLE);
        _testAjaxRegionComponent(Templates.MODALPANEL);
    }

    private void _testAjaxRegionComponent(Templates template) {
        renderPage("/faces/pages/ajaxRegion/ajaxRegionTest.xhtml", template);

        String parentId = getParentId() + "_form:";

        writeStatus("check nested regions");

        String externalLink = parentId + "externalLink";
        String internalLink = parentId + "internalLink";

        String internalElemId = parentId + "internal";
        String externalElemId = parentId + "external";

        selenium.type(internalElemId, "5");
        selenium.type(externalElemId, "5");

        clickById(internalLink);
        waitForAjaxCompletion();
        AssertValueEquals(internalElemId, "5");
        AssertValueEquals(externalElemId, "0");

        selenium.type(internalElemId, "10");
        selenium.type(externalElemId, "10");

        clickById(externalLink);
        waitForAjaxCompletion();
        AssertValueEquals(internalElemId, "10");
        AssertValueEquals(externalElemId, "10");

        writeStatus("verify \"selfRendered\" component's attribute");

        String selfRenderedLink = parentId + "selfRenderedLink";
        clickById(selfRenderedLink);
        waitForAjaxCompletion();
        AssertNotPresent("transientText");

        writeStatus("verify \"renderRegionOnly\" component's attribute");

        String renderOnlyLink = parentId + "renderOnlyLink";
        String renderLink = parentId + "renderLink";

        String renderOnlyElemId = parentId + "renderOnly";
        String renderElemId = parentId + "render";

        selenium.type(renderOnlyElemId, "7");

        clickById(renderOnlyLink);

        AssertValueEquals(renderOnlyElemId, "7");
        AssertValueEquals(renderElemId, "0");

        selenium.type(renderElemId, "11");
        clickById(renderLink);

        AssertValueEquals(renderOnlyElemId, "11");
        AssertValueEquals(renderElemId, "11");
    }

}
