package org.richfaces;

import org.ajax4jsf.test.base.SeleniumTestBase;
import org.ajax4jsf.test.base.Templates;
import org.testng.Assert;
import org.testng.annotations.AfterTest;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.Parameters;
import org.testng.annotations.Test;

public class ComboBoxTest extends SeleniumTestBase {

    public ComboBoxTest() {
        super("http", "localhost", "8080");
    }

    /**
     * This method are invoking before selenium tests started
     */
    @BeforeTest
    @Parameters( { "browser" })
    public void startSelenium(String browser) {
        super.startSelenium(browser);
    }

    /**
     * This method are invoking after selenium tests completed
     */
    @AfterTest
    public void stopSelenium() {
        super.stopSelenium();
    }

    @Test
    public void testComboBoxComponent() throws Exception {
        _testComboBoxComponent(Templates.SIMPLE);
        _testComboBoxComponent(Templates.DATATABLE);
        _testComboBoxComponent(Templates.MODALPANEL);
    }

    private void _testComboBoxComponent(Templates template) {
        renderPage("/faces/pages/comboBox/comboBoxTest.xhtml", template);

        String parentId = getParentId() + "_form:";

        String predefinedCBId = parentId + "predefined";
        String selectItemsCBId = parentId + "selectItems";
        String suggestionValuesCBId = parentId + "suggestionValues";

        writeStatus("check components' default labels");

        getTextById(predefinedCBId + "comboboxField").startsWith("Select a");
        getTextById(selectItemsCBId + "comboboxField").startsWith("Select a");
        getTextById(suggestionValuesCBId + "comboboxField").startsWith("Select a");

        writeStatus("check a combobox with predefined suggestions");

        clickById(predefinedCBId);
        writeStatus("type \"Hunt\"");

        selenium.type(predefinedCBId + "comboboxField", "Hunt");
        selenium.typeKeys(predefinedCBId + "comboboxField", "Hunt");

        Assert.assertTrue(isVisibleById(predefinedCBId + "list"), "Component pop-up should show up on click");

        clickById(predefinedCBId + "list");

        Assert.assertFalse(isVisibleById(predefinedCBId + "list"), "Component pop-up has to be closed");
        Assert.assertEquals(getValue(predefinedCBId), "Hunter");

        delay(2000);
    }

    /**
     * Gets component value.
     * @param clientId
     * @return 
     */
    private String getValue(String clientId) {
        return runScript("$('" + clientId + "comboboxValue').value;");
    }

}
